////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// BeDVI                                                                                                          //
// by Achim Blumensath                                                                                            //
// blume@corona.oche.de                                                                                           //
//                                                                                                                //
// This program is free software! It may be distributed according to the GNU Public License (see COPYING).        //
//                                                                                                                //
// $Id: BeDVI.cc,v 1.3 1996/12/26 15:34:26 achim Exp achim $
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <stdlib.h>
#include <stdio.h>
#include <memory.h>
#include <InterfaceKit.h>
#include "defines.h"

extern "C"
{
  #include "kpathsea/progname.h"
  #include "kpathsea/proginit.h"
  #include "kpathsea/tex-file.h"
};

#include "debug.h"
#include "BeDVI.h"
#include "DVI-Window.h"
#include "DVI-View.h"
#include "FontList.h"

FontList FontData;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// u_long ReadInt(BFile &f, size_t Size)                                                                          //
//                                                                                                                //
// Reads an integer of `Size' bytes from `f'.                                                                     //
//                                                                                                                //
// BFile  &f                            file                                                                      //
// size_t Size                          size of the integer in bytes                                              //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

u_long ReadInt(BFile &f, size_t Size)
{
  u_long x = 0;

  f.Read((u_char *)&x + sizeof(u_long) - Size, Size);

  return x;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// long ReadSInt(BFile &f, size_t Size)                                                                           //
//                                                                                                                //
// Reads a signed integer of `Size' bytes from `f'.                                                               //
//                                                                                                                //
// BFile  &f                            file                                                                      //
// size_t Size                          size of the integer in bytes                                              //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

long ReadSInt(BFile &f, size_t Size)
{
  long x = 0;

  f.Read((u_char *)&x + sizeof(long) - 1, 1);

  if(x & 0x80)
    x |= 0xffffff00;

  x <<= (Size - 1) * 8;

  if(Size > 1)
    f.Read((u_char *)&x + sizeof(long) - Size + 1, Size - 1);

  return x;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// ViewApplication::ViewApplication()                                                                             //
//                                                                                                                //
// initializes the display and global data (see the BeBook).                                                      //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

ViewApplication::ViewApplication():
  BApplication('BDVI'),
  DspInfo(),
  win(NULL),
  vw(NULL),
//  FilePanelMsg(NULL),
  WindowBounds(90.0, 30.0, 630.0, 470.0),
  AntiAliasedDisplay(true)
{
  app_info   AppInfo;
  BFile      AppFile;
  BRecord    *AppRecord;
  record_ref ref;
  u_int      PixelsPerInch = 300;
  u_int      ShrinkFactor  = 3;
  void       *ptr;
  long       len;
  char       Buffer[1024];

  D(bug("ViewApplication::ViewApplication()\n"));

  kpse_set_progname("BeDVI");    // set default name

  // read preferences

  if(GetAppInfo(&AppInfo) != B_NO_ERROR)
    return;

  AppFile.SetRef(AppInfo.ref);
  AppRecord = AppFile.Record();

  if(ptr = AppRecord->FindExtra("BeDVI: win x1", &len))
    memcpy(&WindowBounds.left, ptr, sizeof(WindowBounds.left));

  if(ptr = AppRecord->FindExtra("BeDVI: win y1", &len))
    memcpy(&WindowBounds.top, ptr, sizeof(WindowBounds.top));

  if(ptr = AppRecord->FindExtra("BeDVI: win x2", &len))
    memcpy(&WindowBounds.right, ptr, sizeof(WindowBounds.right));

  if(ptr = AppRecord->FindExtra("BeDVI: win y2", &len))
    memcpy(&WindowBounds.bottom, ptr, sizeof(WindowBounds.bottom));

  if(ptr = AppRecord->FindExtra("BeDVI: anti aliasing", &len))
    memcpy(&AntiAliasedDisplay, ptr, sizeof(AntiAliasedDisplay));

  if(ptr = AppRecord->FindExtra("BeDVI: dpi", &len))
    memcpy(&PixelsPerInch, ptr, sizeof(PixelsPerInch));

  if(ptr = AppRecord->FindExtra("BeDVI: shrink", &len))
    memcpy(&ShrinkFactor, ptr, sizeof(ShrinkFactor));

  SetResolution(PixelsPerInch, ShrinkFactor);

//  if(!(FilePanelMsg = new BMessage(MsgOpenFile)))
//  {
//    DisplayError("Not enough memory!");
//    return;
//  }
//  FilePanelMsg->AddRect("frame",     BRect(120.0, 50.0, 420.0, 350.0));
//  FilePanelMsg->AddRef ("directory", AppInfo.ref);
//
//  getcwd(Buffer, 1024);                             // there apparently is no other way to get the current directory!
//  if(get_ref_for_path(Buffer, &ref) == B_NO_ERROR)
//    FilePanelMsg->ReplaceRef("directory", ref);

  if(!InitAppMenu())
  {
    DisplayError("Not enough memory!");
    return;
  }
  if(!(win = InitWindow()))
  {
    DisplayError("Can't create window!");
    return;
  }
  vw = (DVIView *)win->FindView("DVI-View");

  UpdateMenus();

  win->Show();
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// ViewApplication::~ViewApplication()                                                                            //
//                                                                                                                //
// saves current settings in executable resources (see the BeBook).                                               //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

ViewApplication::~ViewApplication()
{
  app_info AppInfo;
  BFile    AppFile;
  BRecord  *AppRecord;
  double   x;

  D(bug("ViewApplication::~ViewApplication()\n"));

  if(GetAppInfo(&AppInfo) != B_NO_ERROR)
    return;

  AppFile.SetRef(AppInfo.ref);
  AppRecord = AppFile.Record();

  AppRecord->Lock();
  AppRecord->SetExtra("BeDVI: win x1",        &WindowBounds.left,     sizeof(WindowBounds.left));
  AppRecord->SetExtra("BeDVI: win y1",        &WindowBounds.top,      sizeof(WindowBounds.top));
  AppRecord->SetExtra("BeDVI: win x2",        &WindowBounds.right,    sizeof(WindowBounds.right));
  AppRecord->SetExtra("BeDVI: win y2",        &WindowBounds.bottom,   sizeof(WindowBounds.bottom));
  AppRecord->SetExtra("BeDVI: anti aliasing", &AntiAliasedDisplay,    sizeof(AntiAliasedDisplay));
  AppRecord->SetExtra("BeDVI: dpi",           &DspInfo.PixelsPerInch, sizeof(DspInfo.PixelsPerInch));
  AppRecord->SetExtra("BeDVI: shrink",        &DspInfo.ShrinkFactor,  sizeof(DspInfo.ShrinkFactor));
  AppRecord->Commit();
  AppRecord->Unlock();
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// void ViewApplication::ArgvReceived(int argc, char **argv)                                                      //
//                                                                                                                //
// Parses command line arguments (see the BeBook).                                                                //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void ViewApplication::ArgvReceived(int argc, char **argv)
{
  if(argc > 2)
  {
    printf("USAGE: BeDVI [<file>]\n");
    Quit();
    return;
  }
  if(argc == 2)
  {
    record_ref file;

    if(argv[1][0] != '/')
    {
      char *Buffer;
      
      if(!(Buffer = new char[4096]))
      {
        printf("not enough memory!\n");
        Quit();
        return;
      }
      getcwd(Buffer, 4096);
      strncat(Buffer, "/",     4096);
      strncat(Buffer, argv[1], 4096);

      if(get_ref_for_path(Buffer, &file) != B_NO_ERROR)
      {
        delete [] Buffer;
        printf("can't open file!\n");
        Quit();
        return;
      }
      delete [] Buffer;
    }
    else
      if(get_ref_for_path(argv[1], &file) != B_NO_ERROR)
      {
        printf("can't open file!\n");
        Quit();
        return;
      }

    OpenFile(file);
  }
  kpse_set_progname(argv[0]);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// void ViewApplication::RefsReceived(BMessage *msg)                                                              //
//                                                                                                                //
// Loads a file (see the BeBook).                                                                                 //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void ViewApplication::RefsReceived(BMessage *msg)
{
  record_ref file;

  file = msg->FindRef("refs", 0);

  if(file.database >= 0 && file.record >= 0)
    OpenFile(file);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// void ViewApplication::AboutRequested()                                                                         //
//                                                                                                                //
// Displays the About Requester (see the BeBook).                                                                 //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void ViewApplication::AboutRequested()
{
  BAlert *al;

  if(!(al = new BAlert("About",
                       "BeDVI " VERSION " ("__DATE__")\n\n"
                       "by Achim Blumensath\n\n"
                       "This program is free software!\n"
                       "Consult the GNU Public License for more information.",
                       "Ok", NULL, NULL, B_WIDTH_AS_USUAL, B_INFO_ALERT)))
    return;

  al->Go();
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// void ViewApplication::MessageReceived(BMessage *msg)                                                           //
//                                                                                                                //
// Loads the selected file if the FilePanel was closed (see the BeBook).                                          //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void ViewApplication::MessageReceived(BMessage *msg)
{
  switch(msg->what)
  {
    case MsgOpenPanel:
      LaunchFilePanel();
      break;

    case MsgOpenFile:
      D(bug("MsgOpenFile received\n"));
      CloseFilePanel();

      if(msg->HasRef("refs"))
        OpenFile(msg->FindRef("refs"));
      break;

    case MsgAntiAliasing:
      ToggleAntiAliasing();
      break;

    default:
      BApplication::MessageReceived(msg);
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// void ViewApplication::LaunchFilePanel()                                                                        //
//                                                                                                                //
// opens the FilePanel.                                                                                           //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void ViewApplication::LaunchFilePanel()
{
  RunFilePanel(NULL, NULL, NULL, false, NULL);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// void ViewApplication::FilePanelClosed(BMessage *msg)                                                           //
//                                                                                                                //
// stores current settings of the FilePanel (see the BeBook).                                                     //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//void ViewApplication::FilePanelClosed(BMessage *msg)
//{
//  FilePanelMsg->ReplaceRect("frame",     msg->FindRect("frame"));
//  FilePanelMsg->ReplaceRef ("directory", msg->FindRef ("directory"));
//}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// void ViewApplication::OpenFile(record_ref File)                                                                //
//                                                                                                                //
// Loads a file and displays it.                                                                                  //
//                                                                                                                //
// record_ref File                      file to load                                                              //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void ViewApplication::OpenFile(record_ref File)
{
  BFile    f;
  app_info ai;
  DVI      *NewDocument;
  u_long   FileType;
  u_long   FileApp;

  D(bug("ViewApplication::OpenFile(%lu, %lu)\n", File.database, File.record));

  // set icon if not already done

  f.SetRef(File);
  if(f.GetTypeAndApp(&FileType, &FileApp) == B_NO_ERROR)
    if(FileApp == 0)
      if(GetAppInfo(&ai) == B_NO_ERROR)
        f.SetTypeAndApp(MAKE_ID('D', 'V', 'I', ' '), ai.signature);
      else
        f.SetTypeAndApp(MAKE_ID('D', 'V', 'I', ' '), MAKE_ID('B', 'D', 'V', 'I'));

  SetSleepCursor();

  if(!(NewDocument = new DVI(File, &DspInfo)))
  {
    DisplayError("Not enough memory!");
    return;
  }

  if(!NewDocument->Ok())
  {
    delete NewDocument;
    DisplayError("Can't load file!");
    return;
  }

  vw->SetDocument(NewDocument);

  SetNormalCursor();
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// void ViewApplication::DisplayError(char *msg)                                                                  //
//                                                                                                                //
// Displays an Error Requester.                                                                                   //
//                                                                                                                //
// char *msg                            message to be displayed                                                   //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void ViewApplication::DisplayError(char *msg)
{
  BAlert *al;

  if(!(al = new BAlert("Error", msg, "Ok", NULL, NULL, B_WIDTH_AS_USUAL, B_STOP_ALERT)))
    return;

  al->Go();
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// void ViewApplication::SetResolution(u_int PixelsPerInch, u_int ShrinkFactor)                                   //
//                                                                                                                //
// Sets the resolution of the display.                                                                            //
//                                                                                                                //
// u_int PixelsPerInch                  dpi value (0: don't change old value)                                     //
// u_int ShrinkFactor                   factor to shrink document (0: don't change old value)                     //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void ViewApplication::SetResolution(u_int PixelsPerInch, u_int ShrinkFactor)
{
  DspInfo.SetResolution(PixelsPerInch, ShrinkFactor);

  if(PixelsPerInch != 0)                                                       // mode may have changed
  {
    kpse_format_info[kpse_any_glyph_format].path = NULL;                       // clear paths, otherwise kpathseach
    kpse_format_info[kpse_gf_format].path        = NULL;                       // wouldn't update them
    kpse_format_info[kpse_pk_format].path        = NULL;
    kpse_format_info[kpse_vf_format].path        = NULL;

    kpse_init_prog("XDVI", DspInfo.PixelsPerInch, DspInfo.Mode, "cmr10");
    kpse_set_program_enabled(kpse_pk_format,        1, kpse_src_compile);
    kpse_set_program_enabled(kpse_any_glyph_format, 1, kpse_src_compile);
  }

  FontData.FlushShrinkedGlyphes();

  if(vw)
    vw->SetDspInfo(&DspInfo);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// void ViewApplication::ToggleAntiAliasing()                                                                     //
//                                                                                                                //
// Switches between greyscale and monochrome display.                                                             //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void ViewApplication::ToggleAntiAliasing()
{
  AntiAliasedDisplay = !AntiAliasedDisplay;

  FontData.FlushShrinkedGlyphes();

  ((BMenu *)win->FindView("Menu"))->FindItem(MsgAntiAliasing)->SetMarked(AntiAliasedDisplay);

  vw->RedrawBuffer();

  win->Lock();
  vw->Invalidate();
  win->Unlock();
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// void ViewApplication::UpdateMenus()                                                                            //
//                                                                                                                //
// Updates the marked menu items.                                                                                 //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void ViewApplication::UpdateMenus()
{
  BMenu *Menu;
  int   i;

  Menu = ((BMenu *)win->FindView("Menu"));

  for(i = 0; i < NumModes; i++)
    if(strcmp(DspInfo.Mode, ModeNames[i]) == 0)
    {
      Menu->FindItem(ModeDPI[i])->SetMarked(true);
      break;
    }
  Menu->FindItem(DspInfo.ShrinkFactor)->SetMarked(true);
  Menu->FindItem(MsgAntiAliasing)->SetMarked(AntiAliasedDisplay);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// int main()                                                                                                     //
//                                                                                                                //
// main function.                                                                                                 //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int main()
{
  ViewApplication *ViewApp;

  D(bug("BeDVI " VERSION " (" __DATE__ ")\n"));

  if(!(ViewApp = new ViewApplication()))
    exit(1);

  if(!(ViewApp->Ok()))
  {
    delete ViewApp;
    exit(1);
  }

  ViewApp->Run();

  delete ViewApp;

  exit(0);
}
