////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// BeDVI                                                                                                          //
// by Achim Blumensath                                                                                            //
// blume@corona.oche.de                                                                                           //
//                                                                                                                //
// This program is free software! It may be distributed according to the GNU Public License (see COPYING).        //
//                                                                                                                //
// $Id: DVI-Display.cc,v 1.3 1996/12/26 15:34:26 achim Exp achim $
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <InterfaceKit.h>
#include "debug.h"
#include "BeDVI.h"
#include "DVI-Window.h"
#include "DVI-View.h"

static const int  AppMenuLen              = 3;
static const int  NumMenus                = 5;
static const int  NumSubMenus[NumMenus]   = {3, 2, 4, 3, 6};
static const bool RadioModeMenu[NumMenus] = {false, false, false, true, true};

static const struct
{
  char   *Name;
  char   Shortcut;
  bool   Marked;
  u_long Message;
}
AppMenuDef[] =
{
  {"About BeDVI...", '?', false, B_ABOUT_REQUESTED},
  {"Open...",        'O', false, MsgOpenPanel     },
//  {"Save Settings",   0,  false, MsgSaveSettings  },
  {"Quit",           'Q', false, B_QUIT_REQUESTED }
},
MenuDefs[] =
{
  {"File",            0,  false, NULL             },
  {"About...",       '?', false, B_ABOUT_REQUESTED},
  {"Open...",        'O', false, MsgOpenPanel     },
//  {"Save Settings",   0,  false, MsgSaveSettings  },
  {"Quit",           'Q', false, B_QUIT_REQUESTED },
  {"Display",         0,  false, NULL             },
  {"Anti Aliasing",  'A', true,  MsgAntiAliasing  },
  {"Border Line",    'B', false, MsgBorderLine    },
//  {"Measure Window", 'M', false, MsgMeasureWin    },
  {"Move",            0,  false, NULL             },
//  {"Search",         'S', false, MsgSearch        },
  {"First Page",     '<', false, MsgFirst         },
  {"Last Page",      '>', false, MsgLast          },
  {"Next Page",      '+', false, MsgNext          },
  {"Prev Page",      '-', false, MsgPrev          },
  {"Resolution",      0,  false, NULL             },
  {"600 dpi",         0,  false, 600              },
  {"300 dpi",         0,  true,  300              },
  {"100 dpi",         0,  false, 100              },
  {"Magnification",   0,  false, NULL             },
  {"Shrink to 1/1",  '1', false, 1                },
  {"Shrink to 1/2",  '2', false, 2                },
  {"Shrink to 1/3",  '3', true,  3                },
  {"Shrink to 1/4",  '4', false, 4                },
  {"Shrink to 1/5",  '5', false, 5                },
  {"Shrink to 1/6",  '6', false, 6                }
};

static u_short SleepCursorImage[] =
{
  0x1001,  // 16x16 points, depth 1
  0x0907,  // hot spot at (9, 7)
  0x03c0,  // image data
  0x07c0,
  0x0280,
  0x07e0,
  0x1838,
  0x201c,
  0x4026,
  0x4046,
  0x8083,
  0x8103,
  0x8003,
  0x4006,
  0x4006,
  0x200c,
  0x1838,
  0x07e0,
  0x07c0,  // mask data
  0x07c0,
  0x0380,
  0x07e0,
  0x1ff8,
  0x3ffc,
  0x7ffe,
  0x7ffe,
  0xffff,
  0xffff,
  0xffff,
  0x7ffe,
  0x7ffe,
  0x3ffc,
  0x1ff8,
  0x07e0,
};

char SleepPointerNestCnt = 0;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// bool InitAppMenu()                                                                                             //
//                                                                                                                //
// Creates the MainMenu.                                                                                          //
//                                                                                                                //
// Result:                              `true' if successful, otherwise `false'                                   //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool InitAppMenu()
{
  BMessage   *msg;
  BPopUpMenu *Menu;
  BMenuItem  *Item;
  u_int      i;

  D(bug("InitAppMenu()\n"));

  if(!(Menu = new BPopUpMenu("", false, false)))
    return false;

  for(i = 0; i < AppMenuLen; i++)
  {
    if(!(msg = new BMessage(AppMenuDef[i].Message)))
      return false;

    if(!(Item = new BMenuItem(AppMenuDef[i].Name, msg, AppMenuDef[i].Shortcut)))
    {
      delete msg;
      return NULL;
    }
    Item->SetTarget(be_app);
    Item->SetMarked(AppMenuDef[i].Marked);
    Menu->AddItem(Item);
  }

  be_app->SetMainMenu(Menu);

  return true;  
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// static BMenuBar *InitMenu()                                                                                    //
//                                                                                                                //
// Creates the menu of a window.                                                                                  //
//                                                                                                                //
// Result:                              pointer to the menu or `NULL' if an error occures                         //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

static BMenuBar *InitMenu()
{
  BMessage  *msg;
  BMenuBar  *MenuBar;
  BMenu     *Menu;
  BMenuItem *Item;
  u_int     no = 0;
  u_int     i, j;

  D(bug("InitMenu()\n"));

  if(!(MenuBar = new BMenuBar(BRect(0, 0, 0, 0), "Menu")))
    return NULL;

  for(i = 0; i < NumMenus; i++)
  {
    if(!(Menu = new BMenu(MenuDefs[no++].Name)))
      return NULL;
    MenuBar->AddItem(Menu);

    for(j = 0; j < NumSubMenus[i]; j++)
    {
      if(!(msg = new BMessage(MenuDefs[no].Message)))
        return NULL;

      if(!(Item = new BMenuItem(MenuDefs[no].Name, msg, MenuDefs[no].Shortcut)))
      {
        delete msg;
        return NULL;
      }
      Item->SetMarked(MenuDefs[no].Marked);
      Menu->AddItem(Item);
      no++;
    }
    if(RadioModeMenu[i])
      Menu->SetRadioMode(true);
  }
  return MenuBar;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// ViewWindow *InitWindow()                                                                                       //
//                                                                                                                //
// Creates a window.                                                                                              //
//                                                                                                                //
// Result:                              pointer to the window or `NULL' if an error occures                       //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

ViewWindow *InitWindow()
{
  ViewWindow   *win;
  DVIView      *vw;
  BMenuBar     *Menu;
  BTextControl *PageNo;
  BBox         *bx;
  BScrollBar   *sb;
  BRect        MenuR;
  BRect        WindowR;
  BRect        r;

  D(bug("InitWindow()\n"));

  // create window

  if(!(win = new ViewWindow(((ViewApplication *)be_app)->WindowBounds)))
    return NULL;

#ifndef DEBUG

  win->SetDiscipline(FALSE);

#endif

  win->SetSizeLimits(120.0, 1e7, 120.0, 1e7);

  // create menu

  if(!(Menu = InitMenu()))
  {
    win->Quit();
    return NULL;
  }

  win->AddChild(Menu);
  win->SetKeyMenuBar(Menu);

  // get bounds of window and menu

  win->Lock();
  WindowR = win->Bounds();
  MenuR   = Menu->Bounds();
  win->Unlock();

  // create page counter

  r.Set(B_ORIGIN.x, WindowR.bottom - 13.0, B_ORIGIN.x + 49.0, WindowR.bottom + 1.0);

  if(!(bx = new BBox(r, NULL, B_FOLLOW_LEFT | B_FOLLOW_BOTTOM)))
  {
    win->Quit();
    return NULL;
  }
  bx->SetLowColor(127, 127, 127);
  bx->SetViewColor(127, 127, 127);

  r.Set(0.0, 1.0, 50.0, 14.0);

  if(!(PageNo = new BTextControl(r, "PageNumber", NULL, "1", new BMessage(MsgPage), B_FOLLOW_LEFT | B_FOLLOW_BOTTOM)))
  {
    win->Quit();
    return NULL;
  }
  PageNo->SetTarget(win);
  PageNo->SetAlignment(B_ALIGN_LEFT, B_ALIGN_RIGHT);
  PageNo->SetLowColor(127, 127, 127);
  PageNo->SetViewColor(127, 127, 127);
  bx->AddChild(PageNo);
  win->AddChild(bx);

  // create DVI-View

  r.Set(B_ORIGIN.x, MenuR.bottom + 1.0, WindowR.right - 14.0, WindowR.bottom - 14.0);

  if(!(vw = new DVIView(r, "DVI-View")))
  {
    ((ViewApplication *)be_app)->DisplayError("View NULL!");
    win->Quit();
    return NULL;
  }
  if(!vw->Ok())
  {
    ((ViewApplication *)be_app)->DisplayError("View no Ok!");
    delete vw;
    win->Quit();
    return NULL;
  }
  win->AddChild(vw);
  win->Lock();
  vw->MakeFocus(true);
  win->Unlock();

  // create scrollbars

  r.Set(B_ORIGIN.x + 50.0, WindowR.bottom - 13.0, WindowR.right - 13.0, WindowR.bottom + 1.0);

  if(!(sb = new BScrollBar(r, "H-ScrollBar", vw, 0, 0, B_HORIZONTAL)))
  {
    delete vw;
    win->Quit();
    return NULL;
  }
  win->AddChild(sb);

  r.Set(WindowR.right - 13.0, MenuR.bottom, WindowR.right + 1.0, WindowR.bottom - 13.0);

  if(!(sb = new BScrollBar(r, "V-ScrollBar", vw, 0, 0, B_VERTICAL)))
  {
    win->Quit();
    return NULL;
  }
  win->AddChild(sb);

  return win;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// void SetSleepCursor()                                                                                          //
//                                                                                                                //
// Sets the cursor image to a sleeping-cloud.                                                                     //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void SetSleepCursor()
{
  if(SleepPointerNestCnt++ < 1)
    be_app->SetCursor(SleepCursorImage);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// void SetNormalCursor()                                                                                         //
//                                                                                                                //
// Sets the cursor image to the normal Be-hand.                                                                   //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void SetNormalCursor()
{
  if(--SleepPointerNestCnt < 1)
    be_app->SetCursor(B_HAND_CURSOR);
}
