////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// BeDVI                                                                                                          //
// by Achim Blumensath                                                                                            //
// blume@corona.oche.de                                                                                           //
//                                                                                                                //
// This program is free software! It may be distributed according to the GNU Public License (see COPYING).        //
//                                                                                                                //
// $Id: DVI-View.h,v 1.3 1996/12/26 15:34:26 achim Exp achim $
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef VW_VIEW_H
#define VW_VIEW_H

#ifndef _VIEW_H
#include <View.h>
#endif
#ifndef DEFINES_H
#include "defines.h"
#endif
#ifndef DVI_H
#include "DVI.h"
#endif

class DVIView: public BView
{
  private:
    sem_id     DocLock;
    DVI        *Document;

    sem_id     BufferLock;
    BBitmap    *Buffer;
    BView      *BufferView;

    u_int      PageNo;

  public:
            DVIView(BRect frame, const char *Name);
    virtual ~DVIView();

    virtual void Draw(BRect updateRect);
    virtual void FrameResized(float Width, float Height);
    virtual void KeyDown(u_long c);
    virtual void MessageReceived(BMessage *msg);

    void SetDocument(DVI *doc);
    bool SetPage(u_int no);
    bool IncPage();
    bool DecPage();
    void ToggleBorderLine();
    void UpdateWindowSize();
    void UpdateScrollBars();
    void UpdateWindow();
    void RedrawBuffer();
    bool DocumentChanged();
    void SetDspInfo(DisplayInfo *dsp);

    bool Ok()
    {
      return (DocLock    >= B_NO_ERROR) &&
             (BufferLock >= B_NO_ERROR) &&
             (BufferView != NULL);
    }

  private:
    static long SetNoChar(DrawInfo *di, u_char c);
    static long SetChar(DrawInfo *di, u_char c);
    static long SetVFChar(DrawInfo *di, u_char c);
    static void DrawRule(DrawInfo *di, long x, long y, long w, long h);

  friend class DVI;
  friend class Font;
  friend class FontInfo;
};

#endif
