////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// BeDVI                                                                                                          //
// by Achim Blumensath                                                                                            //
// blume@corona.oche.de                                                                                           //
//                                                                                                                //
// This program is free software! It may be distributed according to the GNU Public License (see COPYING).        //
//                                                                                                                //
// $Id: DVI-Window.cc,v 1.3 1996/12/26 15:34:26 achim Exp achim $
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Application.h>
#include <InterfaceKit.h>
#include <stdio.h>
#include "BeDVI.h"
#include "DVI-View.h"
#include "DVI-Window.h"
#include "debug.h"

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// ViewWindow::ViewWindow(BRect frame)                                                                            //
//                                                                                                                //
// Initializes a window (see the BeBook).                                                                         //                                                                                              //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

ViewWindow::ViewWindow(BRect frame): BWindow(frame, "BeDVI " VERSION, B_DOCUMENT_WINDOW, 0) {}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// ViewWindow::~ViewWindow()                                                                                      //
//                                                                                                                //
// Stores the window bounds.                                                                                      //                                                                                              //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

ViewWindow::~ViewWindow()
{
  ((ViewApplication *)be_app)->WindowBounds = Frame();
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// bool ViewWindow::QuitRequested()                                                                               //
//                                                                                                                //
// Closes the application (see the BeBook).                                                                       //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool ViewWindow::QuitRequested()
{
  be_app->PostMessage(B_QUIT_REQUESTED);
  return true;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// void ViewWindow::MessageReceived(BMessage *msg)                                                                //
//                                                                                                                //
// Handles messages from the menu (see the BeBook).                                                               //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void ViewWindow::MessageReceived(BMessage *msg)
{
  switch(msg->what)
  {
    case B_ABOUT_REQUESTED:
    case MsgOpenPanel:
    case MsgAntiAliasing:
      be_app->PostMessage(msg);
      break;

    case MsgMeasureWin:
      break;

    case MsgSearch:
      break;

    case MsgBorderLine:
    case MsgNext:
    case MsgPrev:
    case MsgFirst:
    case MsgLast:
      ((DVIView *)FindView("DVI-View"))->MessageReceived(msg);
      break;

    case MsgSaveSettings:
      break;

    case MsgPage:
    {
      DVIView *vw;
      u_long  no = 1;

      Lock();

      sscanf(((BTextControl *)FindView("PageNumber"))->Text(), "%ld", &no);

      vw = (DVIView *)FindView("DVI-View");
      vw->SetPage(no);
      vw->MakeFocus(true);

      Unlock();
      break;
    }
    default:
      if(msg->what < 10)                                              // shrink factor
        ((ViewApplication *)be_app)->SetResolution(0, msg->what);

      else if(msg->what < 0x10000)                                    // resolution
        ((ViewApplication *)be_app)->SetResolution(msg->what, 0);

      else
        BWindow::MessageReceived(msg);
      break;
  }
}
