////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// BeDVI                                                                                                          //
// by Achim Blumensath                                                                                            //
// blume@corona.oche.de                                                                                           //
//                                                                                                                //
// This program is free software! It may be distributed according to the GNU Public License (see COPYING).        //
//                                                                                                                //
// $Id: PK.cc,v 1.3 1996/12/26 15:34:26 achim Exp achim $
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include<string.h>
#include"debug.h"
#include"BeDVI.h"
#include"Font.h"

static const u_char PK_CmdStart = 240;
static const u_char PK_X1       = 240;
static const u_char PK_X2       = 241;
static const u_char PK_X3       = 242;
static const u_char PK_X4       = 243;
static const u_char PK_Y        = 244;
static const u_char PK_Post     = 245;
static const u_char PK_NOP      = 246;

// local class which contains data shared by the pk-parsing functions

class PKInfo
{
  private:
    Font  *f;
    int   FlagByte;
    u_int InputByte;
    int   BitPos;
    int   DynF;
    int   RepeatCount;

  public:
    PKInfo(Font *fnt): f(fnt), FlagByte(0), InputByte(0), BitPos(0), DynF(0), RepeatCount(0) {}

    bool ReadIndex(DVI *doc);
    void ReadChar(Font *f, u_char c);

  private:
    int  GetNybble();
    int  GetPackedNum();
    bool SkipSpecials();
};

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// void PKInfo::ReadChar(Font *f, u_char c)                                                                       //
//                                                                                                                //
// Reads a character from a font-file.                                                                            //
//                                                                                                                //
// Font   *f                            font                                                                      //
// u_char c                             character                                                                 //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void PKInfo::ReadChar(Font *f, u_char c)
{
  int    i, j;
  int    n;
  int    RowBitPos;
  bool   PaintSwitch;
  u_char *RowStart;
  u_char *cp;
  Glyph  *g;
  long   FPWidth;
  u_long Width, Height;
  u_char Word;
  int    WordWeight;
  int    BytesWide;
  int    RowsLeft;
  int    HBit;
  int    Count;
 
  g = &f->Glyphs[c];

  if(g->UBitMap)
    return;

  FlagByte    = g->FlagByte;
  DynF        = FlagByte >> 4;
  PaintSwitch = ((FlagByte & 8) != 0);
  FlagByte   &= 0x7;

  f->File.Seek(g->Addr, B_SEEK_TOP);

  if(FlagByte == 7)
    n = 4;
  else if(FlagByte > 3)
    n = 2;
  else
    n = 1;

  if(n != 4)
    FPWidth = ReadInt(f->File, 3);
  else
  {
    FPWidth = (long)ReadInt(f->File, 4);
    ReadInt(f->File, 4);
  }
  ReadInt(f->File, n);

  Width  = ReadInt(f->File, n);
  Height = ReadInt(f->File, n);

  g->UBitMap = new BBitmap(BRect(0.0, 0.0, (float)Width - 1.0, (float)Height - 1.0), B_MONOCHROME_1_BIT);

  g->Ux = ReadSInt(f->File, n);
  g->Uy = ReadSInt(f->File, n);

  g->Advance = f->DimConvert * FPWidth;

  if(!g->UBitMap)
    return;

  RowStart = (u_char *)g->UBitMap->Bits();
  cp       = RowStart;

  BytesWide = g->UBitMap->BytesPerRow();
  BitPos    = -1;

  if(DynF == 14)
  {
    bzero(g->UBitMap->Bits(), g->UBitMap->BitsLength());

    for(i = 0; i < Height; i++)
    {
      cp        = RowStart;
      RowStart += BytesWide;

      RowBitPos = 8;

      for(j = 0; j < Width; j++)
      {
        if(--BitPos < 0)
        {
          Word   = ReadInt(f->File, 1);
          BitPos = 7;
        }
        if(--RowBitPos < 0)
        {
          cp++;
          RowBitPos = 7;
        }
        if(Word & (1 << BitPos))
          *cp |= 1 << RowBitPos;
      }
    }
  }
  else
  {
    RowsLeft    = Height;
    HBit        = Width;
    RepeatCount = 0;
    WordWeight  = 8;
    Word        = 0;

    while(RowsLeft > 0)
    {
      Count = GetPackedNum();

      while(Count > 0)
      {
        if(Count < WordWeight && Count < HBit)
        {
          HBit       -= Count;
          WordWeight -= Count;

          if(PaintSwitch)
            Word |= BitMasks[Count] << WordWeight;

          Count = 0;
        }
        else if(Count >= HBit && HBit <= WordWeight)
        {
          if(PaintSwitch)
            Word |= BitMasks[HBit] << (WordWeight - HBit);

          *cp       = Word;
          RowStart += BytesWide;
          cp        = RowStart;

          for(i = RepeatCount * BytesWide; i > 0; i--)
          {
            *cp = *(cp - BytesWide);
            cp++;
          }
          RowStart    = cp;
          RowsLeft   -= RepeatCount + 1;
          RepeatCount = 0;
          Word        = 0;
          WordWeight  = 8;
          Count      -= HBit;
          HBit        = Width;
        }
        else
        {
          if(PaintSwitch)
            Word |= BitMasks[WordWeight];

          *cp++      = Word;
          Word       = 0;
          Count     -= WordWeight;
          HBit      -= WordWeight;
          WordWeight = 8;
        }
      }
      PaintSwitch = 1 - PaintSwitch;
    }
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// int PKInfo::GetNybble()                                                                                        //
//                                                                                                                //
// Reads a nybble from a font-file.                                                                               //
//                                                                                                                //
// Result:                              nybble read                                                               //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int PKInfo::GetNybble()
{
  u_int temp;

  if(BitPos < 0)
  {
    InputByte = ReadInt(f->File, 1);
    BitPos    = 4;
  }
  temp    = InputByte >> BitPos;
  BitPos -= 4;

  return temp & 0xf;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// int PKInfo::GetPackedNum()                                                                                     //
//                                                                                                                //
// Reads a packed number from a font-file.                                                                        //
//                                                                                                                //
// Result:                              number read                                                               //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int PKInfo::GetPackedNum()
{
  int i, j;

  if((i = GetNybble()) == 0)
  {
    for(j = GetNybble(), i++; j == 0; i++)
      j = GetNybble();

    for(; i > 0; i--)
      j = (j << 4) | GetNybble();

    return (j - 15 + ((13 - DynF) << 4) + DynF);
  }
  else
  {
    if(i <= DynF)
      return i;

    if(i < 14)
      return (((i - DynF - 1) << 4) + GetNybble() + DynF + 1);

    if(i == 14)
      RepeatCount = GetPackedNum();
    else
      RepeatCount = 1;

    return GetPackedNum();
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// bool PKInfo::SkipSpecials()                                                                                    //
//                                                                                                                //
// Skips special commands in a font-file.                                                                         //
//                                                                                                                //
// Result:                              `true' if successful, otherwise `false'                                   //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool PKInfo::SkipSpecials()
{
  int i, j;

  do
  {
    FlagByte = ReadInt(f->File, 1);

    if(FlagByte >= PK_CmdStart)
    {
      switch(FlagByte)
      {
        case PK_X1:
        case PK_X2:
        case PK_X3:
        case PK_X4:
          for(i = 0, j = PK_CmdStart; j <= FlagByte; j++)
            i = (i << 8) | ReadInt(f->File, 1);

          f->File.Seek(i, B_SEEK_MIDDLE);
          break;

        case PK_Y:
          ReadInt(f->File, 4);
          break;

        case PK_Post:
        case PK_NOP:
          break;

        default:
          return false;
      }
    }
  }
  while(FlagByte != PK_Post && FlagByte >= PK_CmdStart);

  return true;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// static void ReadChar(Font *f, u_char c)                                                                        //
//                                                                                                                //
// Reads a character from a font-file.                                                                            //
//                                                                                                                //
// Font   *f                            font                                                                      //
// u_char c                             character                                                                 //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

static void ReadChar(Font *f, u_char c)
{
  PKInfo info(f);

  info.ReadChar(f, c);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// bool PKInfo::ReadIndex(DVI *doc)                                                                               //
//                                                                                                                //
// Reads general information from a font-file.                                                                    //
//                                                                                                                //
// DVI *doc                             document the font appears in                                              //
//                                                                                                                //
// Result:                              `true' if successful, otherwise `false'                                   //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool PKInfo::ReadIndex(DVI *doc)
{
  u_long chksum;
  long   hppp, vppp;

  f->ReadChar = ::ReadChar;

  f->File.Seek(ReadInt(f->File, 1), B_SEEK_MIDDLE);

  ReadInt(f->File, 4);

  chksum = ReadInt(f->File, 4);

  if(chksum && f->ChkSum && f->ChkSum != chksum)
    return false;

  hppp = (long)ReadInt(f->File, 4);
  vppp = (long)ReadInt(f->File, 4);

  if(!(f->Glyphs = new Glyph[256]))
    return false;

  while(true)
  {
    u_long BytesLeft;
    int    FlagLowBits;
    u_long c;

    if(!SkipSpecials())
      return false;

    if(FlagByte == PK_Post)
      return true;

    FlagLowBits = FlagByte & 0x7;

    if(FlagLowBits == 7)
    {
      BytesLeft = ReadInt(f->File, 4);
      c         = ReadInt(f->File, 4);
    }
    else if(FlagLowBits > 3)
    {
      BytesLeft = ((FlagLowBits - 4) << 16) + ReadInt(f->File, 2);
      c         = ReadInt(f->File, 1);
    }
    else
    {
      BytesLeft = (FlagLowBits << 8) + ReadInt(f->File, 1);
      c         = ReadInt(f->File, 1);
    }
    f->Glyphs[c].Addr     = f->File.Seek(0, B_SEEK_MIDDLE);
    f->Glyphs[c].FlagByte = FlagByte;

    f->File.Seek(BytesLeft, B_SEEK_MIDDLE);
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// bool ReadPKIndex(DVI *doc, Font *f)                                                                            //
//                                                                                                                //
// Reads general information from a font-file.                                                                    //
//                                                                                                                //
// DVI  *doc                            document the font appears in                                              //
// Font *f                              font                                                                      //
//                                                                                                                //
// Result:                              `true' if successful, otherwise `false'                                   //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool ReadPKIndex(DVI *doc, Font *f)
{
  PKInfo info(f);

  return info.ReadIndex(doc);
}
