////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// BeDVI                                                                                                          //
// by Achim Blumensath                                                                                            //
// blume@corona.oche.de                                                                                           //
//                                                                                                                //
// This program is free software! It may be distributed according to the GNU Public License (see COPYING).        //
//                                                                                                                //
// $Id: list.h,v 1.2 1996/12/26 15:34:26 achim Exp achim $
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef LIST_H
#define LIST_H

#ifndef DEFINES_H
#include"defines.h"
#endif

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// class Node                                                                                                     //
//                                                                                                                //
// class for elements of dubble linked lists.                                                                     //
//                                                                                                                //
// Node *Next              pointer to the next element                                                            //
// Node *Prev              pointer to the previous element                                                        //
//                                                                                                                //
//      Node();                                                                                                   //
//      Node(Node *PrevNode);                                                                                     //
//      ~Node();                                                                                                  //
// Node *Insert(Node *PrevNode);                                                                                  //
// Node *Remove();                                                                                                //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class Node
{
  public:
    Node *Next;
    Node *Prev;

    Node(): Next(NULL), Prev(NULL) {}

    Node(Node *PrevNode)
    {
      Insert(PrevNode);
    }

    virtual ~Node() {}

    Node *Insert(Node *PrevNode)
    {
      Prev = PrevNode;
      Next = Prev->Next;

      Prev->Next = this;
      Next->Prev = this;

      return this;
    }

    Node *Remove()
    {
      Prev->Next = Next;
      Next->Prev = Prev;

      return this;
    }
};

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// Node::Node()                                                                                                   //
//                                                                                                                //
// initializes a node.                                                                                            //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// Node::Node(Node *PrevNode)                                                                                     //
//                                                                                                                //
// initializes a node and inserts it in a list behind `PrevNode'.                                                 //
//                                                                                                                //
// Node* PrevNode          previous node                                                                          //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// Node::~Node()                                                                                                  //
//                                                                                                                //
// does nothing.                                                                                                  //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// void Node::Insert(Node *PrevNode)                                                                              //
//                                                                                                                //
// inserts a node behind `PrevNode' in a list.                                                                    //
//                                                                                                                //
// Node* PrevNode          previous node                                                                          //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// Node *Node::Remove()                                                                                           //
//                                                                                                                //
// removes a node from a list.                                                                                    //
//                                                                                                                //
// Result:                 pointer to the removed node                                                            //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// class List                                                                                                     //
//                                                                                                                //
// class of doubly linked lists.                                                                                  //
//                                                                                                                //
// Node *First             pointer to the first node                                                              //
// Node *PrevNext          always NULL, so ListC consists of two overlapping NodeC-classes                        //
// Node *Last              pointer to the last node                                                               //
//                                                                                                                //
//              List();                                                                                           //
// virtual      ~List();                                                                                          //
// virtual void Clear();                                                                                          //
// virtual bool Empty();                                                                                          //
// virtual List &AddHead(Node *n);                                                                                //
// virtual List &AddTail(Node *n);                                                                                //
// virtual Node *RemHead();                                                                                       //
// virtual Node *RemTail();                                                                                       //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class List
{
  public:
    Node *First;
    Node *PrevNext;
    Node *Last;

    List(): First((Node *)&PrevNext), PrevNext(NULL), Last((Node *)&First) {};

    virtual ~List()
    {
      Clear();
    }

    virtual void Clear()
    {
      while(!Empty())
        delete RemHead();
    }

    virtual bool Empty()
    {
      if((void *)Last != (void *)&First)
        return false;
      else
        return true;
    }

    virtual List &AddHead(Node *n)
    {
      n->Insert((Node *)&First);

      return *this;
    }

    virtual List &AddTail(Node *n)
    {
      n->Insert(Last);

      return *this;
    }

    virtual Node *RemHead()
    {
      return First->Remove();
    }

    virtual Node *RemTail()
    {
      return Last->Remove();
    }

};

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// List::List()                                                                                                   //
//                                                                                                                //
// initializes a list.                                                                                            //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// virtual List::~List()                                                                                          //
//                                                                                                                //
// deletes all nodes in the list.                                                                                 //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// virtual void List::Clear()                                                                                     //
//                                                                                                                //
// deletes all nodes in the list.                                                                                 //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// virtual unsigned short List::Empty()                                                                           //
//                                                                                                                //
// checks if the list is empty.                                                                                   //
//                                                                                                                //
// Result:                 `true' if it is empty, otherwise `false'                                               //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// virtual void List::AddHead(Node *n)                                                                            //
//                                                                                                                //
// inserts a node at the head of a list.                                                                          //
//                                                                                                                //
// Node *n                 pointer to the node                                                                    //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// virtual void List::AddTail(Node *n)                                                                            //
//                                                                                                                //
// inserts a node at the tail of a list.                                                                          //
//                                                                                                                //
// Node *n                 pointer to the node                                                                    //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// virtual Node *List::RemHead()                                                                                  //
//                                                                                                                //
// removes a node from the head of a list.                                                                        //
//                                                                                                                //
// Result:                 pointer to the removed node                                                            //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                //
// virtual Node *List::RemTail()                                                                                  //
//                                                                                                                //
// removes a node from the tail of a list.                                                                        //
//                                                                                                                //
// Result:                 pointer to the removed node                                                            //
//                                                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#endif

